/*
 *  arrays.cpp
 *  CppTutorial
 *
 *  Example showing basic usage of the array data type.
 *
 */

#include <iostream>

int main(int argc, char *argv[])
{
  using std::cout;
  using std::endl;

  // first array example
  {
    // null-terminated character array, a "C-string"
    const char c_arr[] = {'h', 'i', '\0'};
    // a "C-string" constant
    const char *str = "hi";

    // we can print both and their output is identical
    cout << "c_arr = " << c_arr << endl;
    cout << "str = " << str << endl;
  }

  cout << endl;

  // second array example
  {
    unsigned int idx;
    const size_t arr_size=4;

    float f_arr[arr_size];

    // f_arr is the same as &(f_arr[0]), i.e.
    // f_arr points to the array~s first element

    f_arr[0] = 100.0f;
    f_arr[1] = 101.0f;

    // pointer arithmetics; step-size in byte = sizeof(float)
    // let~s assume that sizeof(float) equals 4
    // f_arr+2 increments the address f_arr by 8 bytes, which is
    // exactly the size of 2 float array elements
    *(f_arr+2) = 102.0f; 

    f_arr[3] = *(f_arr+1) + 2.0f; // = 103.0f

    for(idx=0; idx<arr_size; idx++)
      {
	cout << "f_arr[" << idx << "] = " << f_arr[idx] << endl;
      }

  }

} //main()
