/*
 *  pointers_and_references.cpp
 *  CppTutorial
 *
 *  Example showing basic usage of pointers and references
 *  and the important differences between them.
 *
 */

#include <iostream>


int main(int argc, char *argv[])
{
  int i=123, j=456;

  // use "*" for pointer declaration
  int *ptr_to_i = NULL;
  int *ptr2_to_i = &i; // assign i's address to pointer

  // pointers may be uninitialized, but that~s bad practice
  int *ptr3_to_i; 

  // use "&" for reference declaration
  // NOTE: a reference MUST be initialized!
  // int &ref_to_i; // will not compile!!!
  int &ref_to_i = i; 

  ptr_to_i = &i; // make "ptr_to_i" point to "i"

  ptr3_to_i = ptr_to_i; // note that now we don't need the "&"


  // print the values of i and j and addresses of the memory cells, 
  // in which they are stored
  std::cout << "i = " << i << std::endl;
  std::cout << "&i = " << &i << std::endl;
  std::cout << "\n";

  std::cout << "j = " << j << std::endl;
  std::cout << "&j = " << &j << std::endl;
  std::cout << "\n";





  std::cout << "ref_to_i = " << ref_to_i << std::endl;

  // NOTE: prints i's address, not ref_to_i's address!!!
  std::cout << "&ref_to_i = " << &ref_to_i << std::endl; 
  //  std::cout << "*ref_to_i = " << *ref_to_i << std::endl; // won't compile

  std::cout << "\n";





  // "ptr_to_i" itself is stored at:
  std::cout << "ptr_to_i's address " << &ptr_to_i << std::endl;

  std::cout << "ptr_to_i's referenced address " << ptr_to_i << std::endl;

  std::cout << "ptr_to_i's referenced value " << *ptr_to_i << std::endl;

  std::cout << "\n";

  // we can make a pointer point to another location
  ptr_to_i = &j; // now, make "ptr_to_i" point to "j"

  std::cout << "we made ptr_to_i point to j!" << std::endl;

  std::cout << "ptr_to_i's referenced value " << *ptr_to_i << std::endl;

  ptr_to_i = &i; // make "ptr_to_i" point to "i" again

  std::cout << "and now we made ptr_to_i point to i again!" << std::endl;
  std::cout << "ptr_to_i's referenced value " << *ptr_to_i << std::endl;

  std::cout << "\n";




  // POINTER ARITHMETICS:
  // since a pointer has a specific type (except for a void-pointer)
  // the pointer knows,
  // 1.) how many bytes to reference based on a start-address
  // 2.) how many bytes to add to its current address to point to 
  //     the next element of the specified type

  // since here we have an int-pointer, to increment the pointer's
  // referenced address, means to add sizeof(int) bytes!
  std::cout << "ptr_to_i's referenced address before ptr_to_i++ " << ptr_to_i << std::endl;
  ptr_to_i++; 
  std::cout << "ptr_to_i's referenced address after ptr_to_i++ " << ptr_to_i << std::endl;

  std::cout << "sizeof(int) = " << sizeof(int) << std::endl;

  // BEWARE!!!
  // don't dereference "ptr_to_i" now (i.e. don't try to read/write *ptr_to_i)
  // because now "ptr_to_i" is actually not pointing to "i" any more, but
  // to the memory cell stored immediately "after" i... and we don't
  // know, what is stored there (could as well be code - as opposed to data!)!

  ptr_to_i--; // okay, now "ptr_to_i" points to "i" again

  return EXIT_SUCCESS;

} //main()
