/*
 *  simple_classes.cpp
 *  CppTutorial
 *
 */

#include <iostream>

#include "Person.h"
#include "Student.h"

using std::cout;
using std::endl;

//This would not compile if Person::getName were not declared as a const method
void printName(const Person& p) {
	cout << p.getName() << endl;
}

int main(int argc, char* argv[]) {

	//Create a Person
	Person personA("Michael");
	cout << "Person A: " << personA.saySomething() << endl;

	//Create a Student, which is derived from a Person
	Student student("Jo", 1234567);

	//Create a "Person" reference to the student
	Person& person = student;
	
	//This will call Student::saySomething (method override)
	cout << person.saySomething() << endl;
	
	Person* somePerson = new Student("Mic", 2345678);

	//Destroying somePerson has to call Student::~Student() first instead of Person::~Person()
	//Therefore it had to be declared as virtual
	//NOTE: Of course this is also the case with objects created on the stack 
	delete somePerson;
	

	//WATCH OUT, slicing happens
	//Use pointers or references for polymorphism!!
	Student s("Markus", 1234567);
	//Everything defined in Student-scope will be sliced
	Person p = s;
	//Casting down to student (as we assume we are still working with a student-instance)
	Student* studentPtr = static_cast<Student* >(&p);
	//Oops, this will output something really weird, as _matNumber is not defined anymore
	cout << "I am excepting 1234567, but I am getting something really weird: " << studentPtr->getMatNumber() << endl;
	
}
