/*
 *  stack_heap.cpp
 *  CppTutorial
 *
 *  Examples showing usage of stack vs. heap
 */

#include <string>
#include <iostream>

using std::string;
using std::cout;
using std::endl;

int main(int argc, char* argv[]) {

	//The size could be determined also during runtime
	int dynamicArraySize = 10;

	//A float on stack memory
	float stackFloat = 1.0f;
	
	//A float on heap memory
	float* heapFloat = new float(1.0f);
	
	//A float array on the stack (size known at compile time)
	float stackFloatArray[4];
	stackFloatArray[0] = 1.0f;
	stackFloatArray[1] = 2.0f;	
	stackFloatArray[2] = 3.0f;	
	stackFloatArray[3] = 4.0f;			
	
	//A float array on the heap (size can be determined at runtime)
	float* heapFloatArray = new float[dynamicArraySize];
	//new[] does NOT initialize anything, you have to do it yourself
	for (int i = 0; i<dynamicArraySize; i++) {
		heapFloatArray[i] = static_cast<float >( i );
	}	
	
	//Accessing data
	cout << "On stack : " << stackFloat << endl;
	cout << "On heap : " << (*heapFloat) << endl;
	
	cout << "Array on stack: ";
	cout << stackFloatArray[0] << " ";
	cout << stackFloatArray[1] << " ";
	cout << stackFloatArray[2] << " ";
	cout << stackFloatArray[3] << " " << endl;
			
	cout << "Array on heap: ";
	for (int i = 0; i<dynamicArraySize; i++) {
		cout << heapFloatArray[i] << " ";
	}
	
	//Be a good citizen and cleanup
	delete heapFloat;
	delete[] heapFloatArray; //-> don't forget '[]' for arrays!!!	
	
}
