/*
 *  DeepPerson.cpp
 *  CppTutorial
 */

#include "DeepPerson.h"
#include <iostream>
#include <sstream>

using std::cout;
using std::endl;

DeepPerson::DeepPerson() : _name("default-name"), _child(NULL) {

	cout << "DeepPerson: Calling Default Constructor" << endl;
	
}

//We are using a "-clone" suffix to identify the cloned persons.
//If this person has a child, we are creating new memory for the 
//child's clone and copy its data over using the childs Copy Constructor
//Note how this creates a recursion where all data gets copied until a child
//does not have any child.
DeepPerson::DeepPerson(const DeepPerson& rhs) : _name(rhs.getName()+"-clone"), _child(NULL)
{

	cout << "DeepPerson: Calling Copy Constructor" << endl;
	
	//NOTE: that we HAVE to check for NULL in here...
	if (rhs.getChild() != NULL) {
		_child = new DeepPerson(*rhs.getChild());
	}
	
}

DeepPerson::DeepPerson(const string& name) : _name(name), _child(NULL) {

	cout << "DeepPerson: Calling constructor with name " + name << endl;
	
}

DeepPerson& DeepPerson::operator=(const DeepPerson& rhs) {

	cout << "Calling Copy Assignment Operator" << endl;
	
	if (this == &rhs) {
	
		return (*this);
		
	} else {
	
		_name = rhs.getName();
		
		//Same as in Copy Constructor
		if (rhs.getChild() != NULL) {
			_child = new DeepPerson(*rhs.getChild());
		}	
		
		return (*this);
		
	}

}

DeepPerson::~DeepPerson() {

	cout << "DeepPerson " << _name << ": Calling Destructor" << endl;

	//Since we are the owner of the child, we have to take care of it as well
	if (_child != NULL) {
		delete _child;
	}
	
}

DeepPerson * DeepPerson::createChildWithName(const std::string& name) {

	if (_child == NULL) {
	
		_child = new DeepPerson(name);
		return _child;
		
	} else {
	
		//Do appropriate error logging
		cout << "Person " + name + " already has a child." << endl;
		return NULL;
		
	}
	
}

const DeepPerson * DeepPerson::getChild() const {
	return _child;
}

const string& DeepPerson::getName() const {
	return _name;
}

void DeepPerson::setName(const std::string& name) {
	_name = name;
}

string DeepPerson::saySomething() const {
	
	std::ostringstream oss;
	
	oss << _name << " (@ " << this << ") who has";
	
	if (_child != NULL) {
		oss << " a child ";
		oss << _child->saySomething();
	} else {
		oss << " no child.";
	}
	
	return oss.str();
}
