/*
 *  DeepPerson.h
 *  CppTutorial Part 2
 *
 *  This class models a person who can have a child. Further this person is "owner" of the child
 *  in terms of memory and is responsible for allocation and deallocation of its child's data.
 *
 *  This implementation performs a so-called "deep-copy" which means that if this person gets 
 *  copied (or "cloned") its child and child's child (...) will be copied as well.
 *  
 */

#ifndef __DEEP_PERSON_H
#define __DEEP_PERSON_H

#include "TalkativeInterface.h"

#include <string>
#include <sstream>

using std::string;

//Helper method to convert a long to a string
inline string LongToString(long l) {
	std::ostringstream oss;
	oss << l;
	return oss.str();
}

class DeepPerson : public TalkativeInterface {
	
	public:
	
		DeepPerson();
		DeepPerson(const string& name);
		virtual ~DeepPerson();
		
		DeepPerson(const DeepPerson& rhs); //Copy Constructor
		DeepPerson& operator=(const DeepPerson& rhs); //Copy Assignment Operator
		
		const string& getName() const;
		void setName(const std::string& name);

		//This creates a child for this person
		//You do NOT have to deallocate this pointer, as the person itself
		//takes care of it...
		DeepPerson * createChildWithName(const std::string& name);
		const DeepPerson * getChild() const; 

		virtual string saySomething() const;
		
	private:
	
		string _name;
		DeepPerson* _child;

};

#endif //__PERSON_H