/*
 *  Example_Casting.cpp
 *  CppTutorialPart2
 *
 *  Example showing why C++ casts are better than old-style C casts.
 *  Generally: a cleanly designed program should use cast's as little as possible!
 */
 
#include "AmnesicStudent.h"
#include "HealthyStudent.h"
#include <iostream>
#include <string>

using std::cout;
using std::endl;

void checkType(DeepPerson* p) {

	//dynamic_cast returns a NULL ptr if downcast failed
	HealthyStudent* healthyStudentPtr = dynamic_cast<HealthyStudent* >(p);
	if (healthyStudentPtr != NULL) {
		cout << p->getName() << " is instance of HealthyStudent." << endl;
	}
	
	AmnesicStudent* amnesicStudentPtr = dynamic_cast<AmnesicStudent* >(p);
	if (amnesicStudentPtr != NULL) {
		cout << p->getName() << " is instance of AmnesicStudent." << endl;
	}
	
}

int main(int argc, char* argv[]) {

	int a = 7;
	//Use static cast e.g. to convert integral types
	unsigned int b = static_cast<unsigned int >(a);
	//C style cast will compile, but won't represent a double (very dangerous)
	double* p1 = (double*) &a;			
	//C++ style cast is better as the above error would be caught by the compiler
//	double* p2 = static_cast<double*>(&a);	// error
	
	//Use this cast if you have to force the compiler to do illegal things
	//(which you shouldn't, really!)
	double* p2 = reinterpret_cast<double*>(&a);	// ok: I really mean it
	
	const int c = 7;
	//Error, won't compile
	//int* q1 = &c;
	//C style cast would allow this, but it's still invalid code
	int* q2 = (int*)&c;
	//C++ style cast won't allow this, as static_cast doesn't cast away const
	//int* q3 = static_cast<int*>(&c);
	//Use const_cast to cast away const'ness, again
	//you shouldn't have to do this
	int* q4 = const_cast<int*>(&c);
	
	
	//for classes
	DeepPerson* p = new AmnesicStudent("John", 123456);
	
	checkType(p);
	
	delete p;
	
	return EXIT_SUCCESS;
}