/*
 *  Example_Exceptions.cpp
 *  CppTutorialPart2
 *
 *  This example shows how to use exceptions with C++.
 *
 */

#include <iostream>
#include <string>
#include <stdexcept>

using std::string;
using std::exception;
using std::cout;
using std::endl;

void someFunctionA() {

	//We can throw anything
	throw string("Flying message.");
	
}

void someFunctionB() {

	string s("SOME");
	//Oops this will throw out_of_range...
	s.at(4);
	
}

void someFunctionC() {
	
	string* s = new string("SOME");
	
	//Oops, this will throw AND create a memory leak...
	char c = s->at(4);
	
	//Could use shared_ptr in here!
	
	delete s;
	
}

void someFunctionD() throw (std::out_of_range) {
	//This is logically  not correct, 
	//however, it will compile and may terminate during runtime as
	//you throw a string but declared to only throw out_of_range exception
	//NOTE: MSVC is going to ignore the throw guards, anyway..
	throw string("Flying message");
}

int main(int argc, char* argv[]) {
	
	try {
	
		someFunctionA();
		
	} catch (const string& s) {
	
		cout << "Caught a string " << s << endl;
		
	}
	
	try {
	
		someFunctionB();
		
	} catch(const std::out_of_range& e) {
	
		cout << "Got an exception: " << e.what() << endl;
		
	}
	
	try {
	
		someFunctionC();

	} catch (const exception& e) { //catch polymorphically
	
		cout << "Got an exception: " << e.what() << endl;
		
	} catch (...) { //If there is any other exception, this will catch it
		//No exception passes this point!!
	}
	
	//I do not need to catch, if I don't want to...
	//Watch out, this call is going to crash this program!
	//someFunctionD();
	
}