#include <iostream>

#include "DeepPerson.h"
#include "ShallowPerson.h"

#include "AmnesicStudent.h"
#include "HealthyStudent.h"

using std::cout;
using std::endl;

void copyWithShallowPerson() {

	ShallowPerson john("John");
	
	//John is going to have a kid "Johnny"
	//Note that he is supposed to be "the owner" of the child (in terms of memory)
	
	//Johnny in turn, is going to have another kid "Johnny-Lee"
	ShallowPerson* johnny = john.createChildWithName("Johnny");	
	johnny->createChildWithName("Johnny-Lee");
	
	//Prints: 
	//John (@ 0x7fff5fbff1a0) who has a child 
	//		Johnny (@ 0x100100220) who has a child 
	//			Johnny-Lee (@ 0x100100270) who has no child.
	cout << john.saySomething() << endl;
	
	//Now what happens if we copy Johnny?
	ShallowPerson johnCopy = john;
	
	//Prints:
	//John (@ 0x7fff5fbff180) who has a child 
	//		Johnny (@ 0x100100220) who has a child 
	//			Johnny-Lee (@ 0x100100270) who has no child.
	cout << johnCopy.saySomething() << endl;
	
	//Oops, that's not what we intended. Only ONE person can be the parent of ONE child.
	//Exiting this scope will cause the child to be double-deleted... this
	//code is likely to crash.
}

void copyWithDeepPerson() {

	DeepPerson john;
	john.setName("John");
	
	//John is going to have a kid "Johnny"
	//Note that he is supposed to be "the owner" of the child (in terms of memory)
	
	//Johnny in turn, is going to have another kid "Johnny-Lee"
	DeepPerson* johnny = john.createChildWithName("Johnny");	
	johnny->createChildWithName("Johnny-Lee");
	
	//Prints:
	//John (@ 0x7fff5fbff1a0) who has a child 
	//	Johnny (@ 0x100100290) who has a child 
	//		Johnny-Lee (@ 0x100100300) who has no child.
	cout << john.saySomething() << endl;
	
	//Now what happens if we copy Johnny?
	DeepPerson johnCopy(john);
	
	//Prints:
	//John-clone (@ 0x7fff5fbff180) who has a child 
	//	Johnny-clone (@ 0x1001002e0) who has a child 
	//		Johnny-Lee-clone (@ 0x100100350) who has no child.
	cout << johnCopy.saySomething() << endl;
	
	//Oops, that's not what we intended. Only ONE person can be the parent of ONE child.
	//Exiting this scope will cause the child to be double-deleted... this
	//code is likely to crash.
}

int main (int argc, char * const argv[]) {
	
	//NOTE: this is just done for brevity of code examples to fit the slides...
	//		Usually you should still use the std:: namespace
	using std::cout;
	using std::endl;

	copyWithShallowPerson();
	
	copyWithDeepPerson();
	
	//Generally, differentiate between copy assignment and copy constructing
	
	//This will call DeepPerson::DeepPerson(const DeepPerson& rhs)
	DeepPerson somePerson;
	DeepPerson copyConstructed = somePerson;

	DeepPerson copyAssigned;
	//This will class DeepPerson::DeepPerson& operator=(const DeepPerson& rhs);
	copyAssigned = somePerson;
	
	//Let's see how that works with subclasses
	AmnesicStudent aMike("Mike", 123456);
	
	//Prints: "I am Mike, Student no. 123456".
	cout << aMike.saySomething() << endl;
	
	//Copy Mike to Mike-Clone
	AmnesicStudent aMikeClone(aMike);
	
	//Prints:
	cout << aMikeClone.saySomething() << endl;
	
	//Healthy Student knows better...

	HealthyStudent hMike("Mike", 123456);
	
	//Prints: "I am Mike, Student no. 123456".
	cout << hMike.saySomething() << endl;
	
	//Copy Mike to Mike-Clone
	HealthyStudent hMikeClone(hMike);
	
	//Prints: "I am Mike-clone, Student no. 123456"
	cout << hMikeClone.saySomething() << endl;	
	
	return EXIT_SUCCESS;
}
