/*
 *  ShallowPerson.cpp
 *  CppTutorialPart2
 *
 */

#include "ShallowPerson.h"
#include <iostream>
#include <sstream>

using std::cout;
using std::endl;

ShallowPerson::ShallowPerson() : _name("default-name"), _child(NULL) {

	cout << "ShallowPerson: Calling Default Constructor" << endl;
	
}

ShallowPerson::ShallowPerson(const string& name) : _name(name), _child(NULL) {

	cout << "ShallowPerson: Calling constructor with name " + name << endl;
	
}

ShallowPerson::~ShallowPerson() {
	//!!!WARNING: If we were uncommentig the below lines, 
	//this implementation would try to double-delete a pointer after being copied and 
	//destroyed afterwards. There is no way to prevent this from happening without a proper
	//COpy Constructor and Copy Assignment Operator implementation.
	//Therefore we just silently creates memory leaks and do not delete at all.
	//(Or else the demo program would simply crash...)
	//if (_child != NULL)
	//	delete _child;
}

ShallowPerson * ShallowPerson::createChildWithName(const std::string& name) {

	if (_child == NULL) {
	
		_child = new ShallowPerson(name);
		return _child;
		
	} else {
	
		//Do appropriate error logging in here
		return NULL;
		
	}
	
}

const ShallowPerson * ShallowPerson::getChild() const {
	return _child;
}

const string& ShallowPerson::getName() const {
	return _name;
}

void ShallowPerson::setName(const std::string& name) {
	_name = name;
}

string ShallowPerson::saySomething() const {
	
	std::ostringstream oss;
	
	oss << _name << " (@ " << this << ") who has";
	
	if (_child != NULL) {
		oss << " a child ";
		oss << _child->saySomething();
	} else {
		oss << " no child.";
	}
	
	return oss.str();
}
