/*
 *  ShallowPerson.h
 *  CppTutorialPart2
 *
 *  This implementation of a person that is able to have a child has no user implementation
 *  of a Copy Constructor of Copy Assignment Operator. Copying ShallowPerson results in a shallow
 *  copy of its child, i.e. plainly copying the pointer to the child object. Furtermore this implies
 *  further problems (look at its destructor).
 *
 */

#ifndef __SHALLOW_PERSON_H
#define __SHALLOW_PERSON_H

#include <string>
#include "TalkativeInterface.h"

using std::string;

class ShallowPerson : public TalkativeInterface {
	
	public:

		ShallowPerson(); //default constructor
		ShallowPerson(const string& name); //Constructor
		~ShallowPerson();
		
		const string& getName() const;
		void setName(const std::string& name);

		//Note that this person is the owner of the child's data
		ShallowPerson * createChildWithName(const std::string& name);
		const ShallowPerson * getChild() const;

		virtual string saySomething() const;
		
	private:
	
		string _name;
		ShallowPerson* _child;

};

#endif //__SHALLOW_PERSON_H