/*
 *  stl_vectors.cpp
 *  CppTutorialPart2
 *
 *  The example shows some aspects of STL vectors.
 *
 */


#include <iostream>
#include <vector>
#include <string>
#include <stdexcept>

// C interface ... many OpenGL functions look similar!!!
void a_C_param_func(const unsigned int array_len, const float *float_ptr)
{
  unsigned int i=0;

  for(i=0; i<array_len; i++)
    {
      std::cout << "float_ptr[" << i 
		<< "] = " << float_ptr[i] << std::endl;
    }
} //a_C_param_func()


void test_0(void)
{
  float curr_val = 0.0f;

  std::vector<float> float_vec(10);

  std::cout << "in test_0()" << std::endl;
  std::cout << "float_vec.capacity() = " << float_vec.capacity() << std::endl;
  std::cout << "float_vec.size() = " << float_vec.size() << std::endl;

  for(int i=0; i<10; i++, curr_val+=0.1f)
    float_vec.at(i) = curr_val;

  std::cout << "float_vec.capacity() = " << float_vec.capacity() << std::endl;
  std::cout << "float_vec.size() = " << float_vec.size() << std::endl;

  a_C_param_func(float_vec.size(), static_cast<float *>(&(float_vec[0])));

  std::cout << "and again" << std::endl;
  a_C_param_func(float_vec.size(), (&(float_vec[0])));
} //test_0()


void test_1(void)
{
  float curr_val = 0.0f;

  std::vector<float> float_vec;

  std::cout << "in test_1()" << std::endl;
  std::cout << "float_vec.capacity() = " << float_vec.capacity() << std::endl;
  std::cout << "float_vec.size() = " << float_vec.size() << std::endl;

  for(int i=0; i<10; i++, curr_val+=0.1f)
    float_vec.push_back(curr_val);

  std::cout << "float_vec.capacity() = " << float_vec.capacity() << std::endl;
  std::cout << "float_vec.size() = " << float_vec.size() << std::endl;

  a_C_param_func(float_vec.size(), static_cast<float *>(&(float_vec[0])));

} //test_1()


void test_2(void)
{
  float curr_val = 0.0f;

  std::vector<float> float_vec(10);

  std::cout << "in test_2()" << std::endl;
  std::cout << "float_vec.capacity() = " << float_vec.capacity() << std::endl;
  std::cout << "float_vec.size() = " << float_vec.size() << std::endl;

try
  {
  for(int i=0; i<100; i++, curr_val+=0.1f)
    {
      float_vec.at(i) = curr_val; //throws std::out_of_range
      // float_vec[i] = curr_val; //equivalent, but does no idx-checking!
    }
  }
 catch(std::out_of_range &e)
   {
     std::cerr << "oops, out of bounds!!!" << std::endl;
     std::cerr << e.what() << std::endl;
   }

  std::cout << "float_vec.capacity() = " << float_vec.capacity() << std::endl;
  std::cout << "float_vec.size() = " << float_vec.size() << std::endl;


  a_C_param_func(float_vec.size(), static_cast<float *>(&(float_vec[0])));

} //test_2()



int main(int argc, char *argv[])
{

  std::cout << "calling test_0()" << std::endl;
  test_0();
  std::cout << std::endl;


  std::cout << "calling test_1()" << std::endl;
  test_1();
  std::cout << std::endl;


  std::cout << "calling test_2()" << std::endl;
  test_2();
  std::cout << std::endl;


  return EXIT_SUCCESS;

} //main()
