/*
 *  streams.cpp
 *  CppTutorialPart2
 *
 *  The example shows basic C++-stream usage.
 *
 */


#include <iostream>
using std::cout;
using std::endl;

#include <fstream>
#include <sstream>

#include <string>
using std::string;


string read_file(const string &filename)
{
    std::ifstream ifile(filename.c_str());

    return string(std::istreambuf_iterator<char>(ifile),
                  std::istreambuf_iterator<char>());
}

void write_string_to_file(const string &file_name, const string &str)
{
  std::ofstream out_file;

  out_file.open(file_name.c_str());

  // write string "str" to file, just as if you'd
  // write to the command line!
  out_file << str << endl;

  out_file.close();
}


// conversion to string
template<typename T>
std::string to_string(const T &val) {
  std::ostringstream oss;
  oss << val;
  return oss.str();
}

// conversion from string
template<typename T>
T from_string(const std::string &str) {
  std::istringstream iss(str);
  T t;
  iss >> t;
  return t;
}


// conversion from string
template<typename T>
T from_string(const std::string &str, T &t) {
  std::istringstream iss(str);
  iss >> t;
  return t;
}



int main(int argc, char *argv[])
{
  string file_content = read_file("streams.cpp");

  cout << endl;
  cout << " ----------------- file_content = " << endl;
  cout << file_content << endl;
  cout << " ----------------- " << endl;
  cout << endl;

  // "copy" the content of this file into another file...
  write_string_to_file("test_output_file.txt", file_content);
  cout << "wrote this file's content into file \"test_output_file.txt\"" << endl;
  cout << endl;

  int i=123;
  float f=4.567f;
 
  std::ostringstream oss;
  oss << " putting i: " << i;
  oss << " and also f: " << f;
 
  // print the associated string object
  cout << oss.str() << endl;

  cout << endl;
  cout << endl;

  // now test the conversion methods
  
  cout << "to_string(124) = " << to_string(124) << endl;

  std::string conversion_to_string = to_string(i);


  conversion_to_string = to_string(i);

  cout << "i = " << i << endl;
  cout << "conversion_to_string.c_str() = " << conversion_to_string.c_str() << endl;

  // NOTE: we need the <int>, otherwise the compiler cannot deduce the type!
  int j = from_string<int>(conversion_to_string);
  cout << "j = " << j << endl;

  // NOTE: we do not need the <int>, since the compiler can deduce the type from the 2nd argument!
  int k = from_string(conversion_to_string, k);
  cout << "k = " << k << endl;

  int l;
  from_string(conversion_to_string, l); //call-by-reference => no need to assign return value explicitly
  cout << "l = " << l << endl;

  int z;
  from_string("54321", z); //call-by-reference => no need to assign return value explicitly
  cout << "z = " << z  << endl;
  cout << "z + 1 = " << z + 1 << endl; // OK, it is really a number, not a string!

  cout << endl;

  conversion_to_string = to_string(f);

  cout << "f = " << f << endl;
  cout << "conversion_to_string.c_str() = " << conversion_to_string.c_str() << endl;

  float g = from_string<float>(conversion_to_string);
  cout << "g = " << g << endl;

  float h = from_string(conversion_to_string, h);
  cout << "h = " << h << endl;



  return EXIT_SUCCESS;
} //main()
