/*
 *  templates.cpp
 *  CppTutorialPart2
 *
 *  The example shows a simple generic function and how
 *  to implement it using C++-templates.
 *
 */

#include <iostream>
using std::cout;
using std::endl;

#include <string>


int my_max(int val1, int val2)
{
  cout << "int max" << endl;
 
  if(val1 >= val2)
    return val1;
  else
    return val2;
}


//we could add many overladed versions of my_max for many types!!!
/*
float my_max(float val1, float val2)
{
  cout << "float max" << endl;

  return (val1 >= val2) ? val1 : val2;
}

std::string my_max(std::string val1, std::string val2)
{
  cout << "string max" << endl;

  return (val1 >= val2) ? val1 : val2;
}
*/


template<typename T>
T my_generic_max(T val1, T val2)
{
  cout << "generic max" << endl;

  if(val1 >= val2)
    return val1;
  else
    return val2;
}

template<> const char *my_generic_max<const char *>(const char *val1, const char *val2)
{
  cout << "--- template specialized for C-strings!" << endl;
  if(std::strcmp(val1, val2) >= 0)
    return val1;
  else
    return val2;
}


int main(int argc, char *argv[])
{
  int i_val1 = 4;
  int i_val2 = 5;

  float f_val1 = 4.5f;
  float f_val2 = 5.5f;

  std::string s_val1("four");
  std::string s_val2("five");

  // ==> int , int  
  cout<<"my_max("<<i_val1<<","<<i_val2<<")="<<my_max(i_val1,i_val2)<<endl;

  // ==> float, float
  // if the float-version of my_max is not defined, 
  // the maximum float value will be cast to an integer!
  // at least the call will work...
  cout<<"my_max("<<f_val1<<","<<f_val2<<")="<<my_max(f_val1,f_val2)<<endl; 

  // ==> int, float
  // "ambiguity" is implicitly solved by float-to-int cast!  
  // ambiguity is present, if we defined the "float-version"
  // float my_max(float val1, float val2)
  cout<<"my_max("<<i_val1<<","<<f_val2<<")="<<my_max(i_val1,f_val2)<<endl;

  // ==> string, string
  // will fail, if not defined for strings!!!
  // no implicit cast possible
  //  cout<<"my_max("<<s_val1<<","<<s_val2<<")="<<my_max(s_val1,s_val2)<<endl;


  cout << endl;

  // ==> int, int
  cout<<"my_generic_max("<<i_val1<<","<<i_val2<<")="<<my_generic_max(i_val1,i_val2)<<endl;

  // ==> float, float
  cout<<"my_generic_max("<<f_val1<<","<<f_val2<<")="<<my_generic_max(f_val1,f_val2)<<endl;

  // ==> int, float
  //ambiguity!  cout<<"my_generic_max("<<i_val1<<","<<f_val2<<")="<<my_generic_max(i_val1,f_val2)<<endl;

  // resolve type ambiguity through explicit template instantiation
  cout<<"my_generic_max<int>("<<i_val1<<","<<f_val2<<")="<<my_generic_max<int>(i_val1,f_val2)<<endl;
  cout<<"my_generic_max<float>("<<i_val1<<","<<f_val2<<")="<<my_generic_max<float>(i_val1,f_val2)<<endl;

  // ==> string, string
  cout<<"my_generic_max("<<s_val1<<","<<s_val2<<")="<<my_generic_max(s_val1,s_val2)<<endl;


  /*
  cout << "and now with C-strings" << endl; //only compares addresses!
  cout<<"my_generic_max("<<s_val1<<","<<s_val2<<")="<<my_generic_max(s_val1.c_str(),s_val2.c_str())<<endl;
  cout<<"my_generic_max("<<s_val2<<","<<s_val1<<")="<<my_generic_max(s_val2.c_str(),s_val1.c_str())<<endl;
  */


  const char *s1 = "Vienna";
  const char *s2 = "Baden";

  // without specialized version of template for C-strings, prints: 
  // my_generic_max(Vienna,Baden)=Baden
  // because only the string's addresses are compared, not the strings
  // themselves!
  cout<<"my_generic_max("<<s1<<","<<s2<<")="<<my_generic_max(s1,s2)<<endl;
  /*
    --- template specialized for C-strings!
    now it works great and really compares the strings lexicographically  
    my_generic_max(Vienna,Baden)=Vienna
  */

  return EXIT_SUCCESS;
}
